--                        -*- Mode: lua; lua-indent-level: 2; tab-width: 2 -*-
--[[
LibFilter_ItemRack

Author: Daphne Pfister <tiiroib02@sneakemail.com>

Thanks to Gello for writing ItemRack

License

Copyright (c) 2008 Daphne Pfister

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
]]

local MAJOR, MINOR = "LibFilter-item-ItemRack", tonumber(("$Revision: 17 $"):match("(%d+)"))
local filter = LibStub:NewLibrary(MAJOR, MINOR)
if not filter then return end

-- Avoid Global Lookups
local _G          = getfenv(0)
local assert      = _G.assert
local ipairs      = _G.ipairs
local next        = _G.next
local pairs       = _G.pairs
local select      = _G.select
local string      = _G.string
local strjoin     = _G.strjoin
local table       = _G.table
local type        = _G.type
local unpack      = _G.unpack

local strmatch    = string.match
local tinsert     = table.insert
local tsort       = table.sort

-- No need for a full blown locale library here
local L
do
	local LL = {
		enUS = {
			["ItemRack Set:%s"] = true,
			["Unknown"] = true,
			["ItemRack Sets"] = true,
			["Set Name"] = true,
			["Any"] = true,
			["None"] = true,
			["ItemRack"] = true,
			["Filter by ItemRack set"] = true,
			["Fuzzy match"] = true,
			["Match by the item number, ignoring enchantments, variations, armor kits, etc."] = true,
		},
		koKR = {
			["ItemRack Set:%s"] = "ItemRack 세트명:%s",
			["Unknown"] = "가격미확인", -- check
			["ItemRack Sets"] = "ItemRack 세트",
			["Set Name"] = "세트명",
			["Any"] = "모두",
			["None"] = "없음", -- check
			--["ItemRack"] = true,
			["Filter by ItemRack set"] = "ItemRack 세트에 따라 필터링",
			["Fuzzy match"] = "미세분화 분류",
			["Match by the item number, ignoring enchantments, variations, armor kits, etc."] = "마법부여나 키트 효과등을 무시하고 아이템 ID에 따라 분류합니다",
		},
		-- Based on swix's translation for ClosetGnome
		roRU = {
			["ItemRack Set:%s"] = "Набор ItemRack:%s",
			["Unknown"] = "Неизвестно",
			["ItemRack Sets"] = "Наборы ItemRack",
			["Any"] = "Любой",
			["None"] = "Ни один",
			["ItemRack"] = "ItemRack",
			["Filter by ItemRack set"] = "Фильтр по наборам ItemRack",
			["Fuzzy match"] = "Нечёткое соответствие",
			["Match by the item number, ignoring enchantments, variations, armor kits, etc."] = "Соответствие по номеру вещи, игнорирование зачарований, вариаций, коплектов брони и т.п.",
		},
	}

	local gGameLocale = GetLocale()
	local enL = LL["enUS"]
	L = LL[gGameLocale] or LL["enUS"]

	for vKey,vValue in pairs(enL) do
		if not L[vKey] then L[vKey] = vValue end
	end
	for vKey,vValue in pairs(L) do
		if vValue == true then L[vKey] = vKey end
	end
end

filter.author = "Nayala"
filter.credits = "ItemRack was written by Gello"
filter.description = L["Filter by ItemRack set"]
filter.filtertype = "item"
filter.key = "ItemRack"
filter.name = L["ItemRack"]

filter.callbacks = filter.callbacks or LibStub("CallbackHandler-1.0"):New(filter)
filter.events = filter.events or {}
filter.myFrame = filter.myFrame or CreateFrame("Frame","LibFilter_ItemRack_Frame")
local callbacks = filter.callbacks
local events = filter.events
local myFrame = filter.myFrame

local gItemsTable = {}
local gSetsTable = {}

-- Prototypes
local LinkToIdsAndItemNumber -- (pLink) -- returns id, idbug and fuzzy
local GetPlanItemNumber -- (pId)
local CacheSetItem -- (pSet, pItem, pTable1, pTable2, ...)
local CacheSets -- ()
local UpgradeSettings -- (pSettings)
local ExpandSettings -- (pSettings)
local NormalizeSettings -- (pSettings)
local Update
local HookItemRackFunc -- (pTable,pFunc,pTarget)
local HookItemRack
local UpdateNextFrame
local ToggleSettingsSet -- (pSettings, pSet)
local ToggleSettingsFuzzy --- (pSettings)

function LinkToIdsAndItemNumber(pLink)
	if type(pLink) == "string" then
		return strmatch(pLink,"(((%-?%d+):%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+):%-?%d+)")
	end
	return nil,nil,nil
end

function GetPlanItemNumber(pId)
	if type(pId) == "string" then
		return strmatch(pId,"(%-?%d+)")
	else
		return pId
	end
end

function CacheSetItem(pSet, pItem, ...)
	if pItem then
		local vItemNumber = GetPlanItemNumber(pItem)

		for i=1,select("#",...) do
			local vCurrentTable = select(i,...)

			if vCurrentTable then
				if not vCurrentTable[pItem] then vCurrentTable[pItem] = {} end
				vCurrentTable[pItem][pSet] = true

				if vItemNumber ~= pItem then
					if not vCurrentTable[vItemNumber] then vCurrentTable[vItemNumber] = {} end
					vCurrentTable[vItemNumber][pSet] = true
				end
			end
		end
	end
end

function CacheSets()
	-- Normal sets
	local vSets = ItemRackUser.Sets

	gItemsTable = {}
	gSetsTable = {}
	for vSetName, vSet in pairs(vSets) do
		if not strmatch(vSetName,"^~") then
			gSetsTable[vSetName] = true
			for slot=0,19 do
				CacheSetItem(vSetName,LinkToIdsAndItemNumber(vSet.equip[slot]),gItemsTable)
			end
		end
	end

	-- Queues
	local vQueues = ItemRackUser.Queues

	local vSetName = "Queue"
	gSetsTable[vSetName] = true
	for vSlot, vQueue in pairs(vQueues) do
		for vOrder,vSetItem in ipairs(vQueue) do
			if vSetItem == 0 then break end
			CacheSetItem(vSetName,LinkToIdsAndItemNumber(vSetItem),gItemsTable)
		end
	end
end

function UpgradeSettings(pSettings)
	local vSets = pSettings.sets
	if not vSets then
		local vSet = pSettings.set
		if not vSet or vSet == "Any" then
			vSets = { Any = true }
		elseif vSet == "None" then
			vSets = { }
		else
			vSets = { [vSet] = true }
		end
		pSettings.sets = vSets
	end
	return vSets
end

function ExpandSettings(pSettings)
	local vSets = pSettings.sets or UpgradeSettings(pSettings)

	if vSets.Any then
		for vSet in pairs(gSetsTable) do
			vSets[vSet] = true
		end
	end
	return vSets
end

function NormalizeSettings(pSettings)
	local vSets = pSettings.sets or UpgradeSettings(pSettings)

	local vNextSet = next(vSets)

	-- No sets selected
	if not vNextSet then
		pSettings.set = "None"
		return vSets
	end

	-- Just one set selected
	if not next(vSets,vNextSet) then
		pSettings.set = vNextSet
		return vSets
	end

	-- Okay check if all sets are selected
	vSets.Any = true
	for vSet in pairs(gSetsTable) do
		if not vSets[vSet] then vSets.Any = nil end
	end

	if vSets.Any then
		vSets = { Any = true }
		pSettings.sets = vSets
		pSettings.set = "Any"
		return vSets
	end

	pSettings.set = "????"
	return vSets
end

myFrame:SetScript("OnEvent",function (self, event, ...) events[event](...) end)

function Update()
	callbacks:Fire("FilterUpdate")
end

function filter:CacheSetsAndUpdate()
	CacheSets()
	-- Should really only fire cache if there were changes
	Update()
end

filter.origCalls = filter.origCalls or {}
filter.hooked = filter.hooked or {}
local gOrigCalls = filter.origCalls
local gHooked = filter.hooked

function HookItemRackFunc(pTable,pFunc,pTarget)
	if not pTable then return end
	if not gHooked[pFunc] then
		local vOrig = pTable[pFunc]
		gOrigCalls[pFunc] = vOrig
		pTable[pFunc] = function(...)
											vOrig(...)
											filter[pTarget](filter,...)
										end
		gHooked[pFunc] = pTarget
	end
end

function filter:SaveSet()
	filter:CacheSetsAndUpdate()
end

function filter:DeleteSet()
	filter:CacheSetsAndUpdate()
end

function filter:InitCore()
	filter:CacheSetsAndUpdate()
end

local gOptionsHooked
function filter:HookOptions()
	if gOptionsHooked then return end
	if not ItemRackOpt then return end
	gOptionsHooked = true

	HookItemRackFunc(ItemRackOpt,"SaveSet","SaveSet")
	HookItemRackFunc(ItemRackOpt,"DeleteSet","DeleteSet")
end

local gIsHooked = nil
local gNeedCache = nil

function HookItemRack()
	if not ItemRack then return end
	if gIsHooked then return end
	gIsHooked = true

	if ItemRack_Version and ItemRack_Version < 2 then
		assert("LibFilter_ItemRack; Unsupported version of ItemRack")
	end

	HookItemRackFunc(ItemRack,"InitCore","InitCore")
	if ItemRackOpt then
		filter:InitOptions()
	else
		HookItemRackFunc(ItemRack,"ToggleOptions","HookOptions")
	end
	myFrame:UnregisterEvent("ADDON_LOADED")
	gNeedCache = true
	UpdateNextFrame()
end

function events.UPDATE()
	myFrame:SetScript("OnUpdate",nil)
	if gNeedCache then
		CacheSets()
		gNeedCache = nil
	end
	HookItemRack()
end

function UpdateNextFrame()
	myFrame:SetScript("OnUpdate",function (self, ...) events.UPDATE(...) end)
end

function events.ADDON_LOADED()
	UpdateNextFrame()
end

function events.PLAYER_LOGIN()
	UpdateNextFrame()
	myFrame:RegisterEvent("ADDON_LOADED")
end

if IsLoggedIn() then
	UpdateNextFrame()
else
	myFrame:RegisterEvent("PLAYER_LOGIN")
end

function filter:NewSettings(pSettings)
	local vRes = {
		Any = pSettings and pSettings.Any or false,
		set = pSettings and pSettings.set,
		fuzzy = pSettings and pSettings.fuzzy or false,
	}
	if pSettings and pSettings.sets then
		local vSets = {}
		for vKey,vValue in pairs(pSettings.sets) do
			vSets[vKey] = vValue
		end
		vRes.sets = vSets
	else
		UpgradeSettings(vRes)
	end
	return vRes
end

function filter:FreeSettings(pSettings)
end

function filter:DescribeSettings(pSettings)
	local vSets = pSettings.sets or UpgradeSettings(pSettings)
	local vSetList
	if not next(vSets) then
		vSetList = L["None"]
	elseif vSets.Any then
		vSetList = L["Any"]
	else
		local vSetNames = {}
		for set in pairs(vSets) do
			tinsert(vSetNames,set)
		end
		tsort(vSetNames)
		vSetList = strjoin(", ",unpack(vSetNames))
	end

	local vDescription = L["ItemRack Set:%s"]:format(vSetList)
	if pSettings.fuzzy then
		vDescription = vDescription.." "..L["Fuzzy match"]
	end
	return vDescription
end

function ToggleSettingsSet(pSettings, pSet)
	if pSet == "Any" then
		pSettings.set = "Any"
		pSettings.sets = { Any = true }
	elseif pSet == "None" then
		pSettings.set = "None"
		pSettings.sets = { }
	else
		local vSets = ExpandSettings(pSettings)

		-- Okay update the given set
		vSets[pSet] = not vSets[pSet] and true or nil

		-- Okay normalize
		vSets.Any = nil -- Since we are expanded, can clear Any
		NormalizeSettings(pSettings)
	end
end

function toggleSettingsFuzzy(pSettings)
	pSettings.fuzzy = not pSettings.fuzzy
end

function filter:SettingsOptions(pSettings,pTable,pMethod)
	local vCallback = Update

	if pTable then
		if type(pTable) == "string" then
			vCallback = function() pTable[pMethod](pTable,pSettings) end
		else
			vCallback = function() pTable(pSettings) end
		end
	end

	local vArgs = {
		{
			order = 1,
			type = "header",
			name = L["ItemRack Sets"],
		},
		{
			order = -1,
			type = "toggle",
			name = L["Fuzzy match"],
			desc = L["Match by the item number, ignoring enchantments, variations, armor kits, etc."],
			get = function() return pSettings.fuzzy end,
			set = function(pV)
							ToggleSettingsFuzzy(pSettings)
							vCallback()
						end,
		},
		{
			order = 2,
			type = "toggle",
			name = L["Any"],
			desc = L["Any"],
			get = function() return pSettings.sets.Any end,
			set = function(pV)
							ToggleSettingsSet(pSettings,"Any")
							vCallback()
						end,
		},
		{
			order = 3,
			type = "toggle",
			name = L["None"],
			desc = L["None"],
			get = function() return not next(pSettings.sets) end,
			set = function(pV)
							ToggleSettingsSet(pSettings,"None")
							vCallback()
						end,
		},
	}
	local vSetNames = {}
	for vSet in pairs(gSetsTable) do
		tinsert(vSetNames,vSet)
	end
	tsort(vSetNames)
	local vOrder = 4
	for _,vSet in ipairs(vSetNames) do
		vOrder = vOrder + 1
		vArgs[vOrder] = {
			type = "toggle",
			order = vOrder,
			name = vSet,
			desc = vSet,
			get = function() return pSettings.sets[vSet] or pSettings.sets.Any end,
			set = function(pV)
							ToggleSettingsSet(pSettings,vSet)
							vCallback()
						end,
		}
	end
	return { type = "group", desc = "LibFilter_Outfitter options", args = vArgs }
end

function filter:TestIfMatch(settings,link)
	local id,idbug,idfuzzy = LinkToIdsAndItemNumber(link)
	if not id then return end
	if not idbug then idbug = id end

	local sets = settings.sets or UpgradeSettings(settings)
	local any = sets.Any
	local none = not next(sets)

	if settings.fuzzy then
		id = idfuzzy
		idbug = idfuzzy
		if not id then return end
	end

	local res = gItemsTable
	if res then
		if id and res[id] then
			res = res[id]
		elseif idbug and res[idbug] then
			res = res[idbug]
		else
			res = nil
		end
	end
	if none then
		return not res
	elseif any or not res then
		return res
	else
		for set in pairs(sets) do
			if res[set] then return res end
		end
	end
	return nil
end
